;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x01e1838f45f489241520c0984d543889 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xe3bddca40010a98b63e48f5c16a94cb7 . "./code.cg2js/cg-printing.fasl")
(#xbcfaa774b890c6f5b2ce7ee25de4d1e4 . "./code.cg2js/cg-caret.fasl")
(#xf50515e184d794d795c39ab1d155abf5 . "./code.cg2js/cg-item-list.fasl")
(#x6e93ea8283925c341d3f365957e036fe . "./code.cg2js/cg-progress-indicator.fasl")
(#x743c409bdfa6a889dde1dc61f8737b50 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x9fa6478b004407e5a6cf73ce2f3f9f1c . "./code.cg2js/cg-outline.fasl")
(#x3a64576e03ceb72c3d9792f6c68a6956 . "./code.cg2js/cg-palette.fasl")
(#x397e0befd3a79b2bd10edf5ea52cbaeb . "./code.cg2js/cg-selected-object.fasl")
(#xce37e0643ada4b49667b314b6dc73c66 . "./code.cg2js/cg-status-bar.fasl")
(#x003dc149a5746dcb83f2ad73b4aba693 . "./code.cg2js/cg-wrap-string.fasl")
(#x9e2dd567e97031ffc5d65dd38687a43d . "./code.cg2js/cg-fill-texture.fasl")
(#x43dda2d8fb5f170c6f258f648cb90409 . "./code.cg2js/cg-layout.fasl")
(#x4acea299eb042999aae39dcb6b87c71f . "./code.cg2js/cg-timer.fasl")
(#x9b845d4df6ff46fa697d1daefce4eb13 . "./code.cg2js/cg-dialog-item.fasl")
(#x15b07784a06876a74e909007417f381a . "./code.cg2js/cg-button.fasl")
(#xecd46766790b0169bc560a33968d85a6 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x3064fe398282256478a71958a5b8be2b . "./code.cg2js/cg-tray.fasl")
(#xf2843e27336b9208cf8ddf5395676b6e . "./code.cg2js/cg-plot-widget.fasl")
(#x7baa1dca7c6ce88c6913ba35af424aac . "./code.cg2js/cg-pixmap.fasl")
(#x43e68b4634cf5b5fbfcf1e98843a706b . "./code.cg2js/cg-font-dialog.fasl")
(#x3d9d8b18f32c1f28e87a2f01041b0af9 . "./code.cg2js/cg-os-widget.fasl")
(#x8ed46c18b6253b3465658a4a468d79bb . "./code.cg2js/cg-text-or-combo.fasl")
(#x70ad8c575b3ef60df34c8f8e0b87b14f . "./code.cg2js/cg-trackbar.fasl")
(#xa9f686b5e02140834e1e0dcee4e721ba . "./code.cg2js/cg-up-down-control.fasl")
(#xa6e983c3826172362020931557f590fd . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x2856d289241230e633e19ac6f702c078 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x3365aa4f858e739edeb19504aecb0754 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x756616ead69ff224588da4b39c833ca1 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xdce7e83a87d63b291d84140e82032d5c . "./code.cg2js/cg-chart-widget.fasl")
(#x5acd84cca3fe1bc489473f2518f880fa . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf3017fcf88fe25cf861d2754b988f026 . "./code.cg2js/cg-group-box.fasl")
(#x7dc4c53f1279fe0639da79daa92cb530 . "./code.cg2js/cg-static-text.fasl")
(#xa78e2335f4ecbbd73e4ff5af3ca1ab95 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x292331589620456a6f6e5bf6b6a8b4b5 . "./code.cg2js/cg-color-dialog.fasl")
(#x6ee70e66225bd136ea85b92729975a9a . "./code.cg2js/cg-intersect.fasl")
(#x41334ff286cf8435c2e4777cfa278262 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xfea03664a39175ba0d0ce41dfeb4509d . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xc54a37a81e695d1e35ffd90643f4b494 . "./code.cg2js/cg-html-dialog.fasl")
(#xd99d5e8200e6f70637803e3f2beb57c7 . "./code.cg2js/cg-lettered-menu.fasl")
(#x891bab60339eb78e83c4bc0bb54dbede . "./code.cg2js/cg-class-grid.fasl")
(#xf2212196a6fb2ceb9a41546d0e338793 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd3c8774c70049504669c7cc6adf371a4 . "./code.cg2js/cg-calendar.fasl")
(#xc060f4a7bfb9bdf7511b71ff41e3eebf . "./code.cg2js/cg-editable-text.fasl")
(#xa8cf515f99229fc68c47b8bef72cf47d . "./code.cg2js/cg-property.fasl")
(#x31035663222179cda0caf7209c94529b . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xc58378f6c116e74215e3b8f9bdb77372 . "./code.cg2js/cg-grid-widget.fasl")
(#xd95aac8270d76d55c0549cdc146fbe23 . "./code.cg2js/cg-paren-matching.fasl")
(#x0c67450fead3d3a5c8fd0461fe77dff4 . "./code.cg2js/cg-dev.fasl")
(#xb49e4fcd739d3196c4c430059c64bd07 . "./code.cg2js/cg-drawable.fasl")
(#x53a42b8c416b251fa93c103240323844 . "./code.cg2js/cg-curve.fasl")
(#x962f24ee8c3113a490edca82c684a548 . "./code.cg2js/cg-lisp-widget.fasl")
(#x8ed8d208bbef6f6076d9bf47b7170dee . "./code.cg2js/cg-dde-utils.fasl")
(#x1ccaef9099a6334bfc6d7a595d55d571 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x1b7e5dd6af8c74b7e3cc0ffb2dd1438f . "./code.cg2js/cg-object-editor-layout.fasl")
(#x558dad9e8763f9a3c4e7e17714aaaa30 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x8ef3a766ddc0966278550caa9705f3a1 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xfd83c1a467a192ea237cbcbc3db5dcc3 . "./code.cg2js/cg-lamp.fasl")
(#xda7de0e4e5885baa85734269c2d3c9ec . "./code.cg2js/cg-common-control.fasl")
(#x4df79c791a367b8cda77481569eb815c . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xd46a252aee9f6553f3a4023370d60450 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe0d59f440d402dc9e5fc68058d5673fe . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xedd5f3824bc1100f999a41e4fb544ebe . "./code.cg2js/cg-project-window.fasl")
(#x24f9356d8557dc7d9c822df9f5f8c933 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x5f2e13aef17ed2a353acaade57143efb . "./code.cg2js/cg-class-support.fasl")
(#xecc17c5358f89ecd2d813cbe1a2382af . "./code.cg2js/cg-multi-picture-button.fasl")
(#xacf19f4f4becd3217deb96072ef3f6a5 . "./code.cg2js/cg-yes-no-list.fasl")
(#xc9b745d43f187293f7678c3ade3776d9 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x9a1d0d2af552408a34250b0864009b33 . "./code.cg2js/cg-string-dialog.fasl")
(#x7317b985f93886911f5b143ec4ee8076 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xfb61281f6ca6a10d16f475a9a7021e1b . "./code.cg2js/cg-dropping-outline.fasl")
(#xa8c4f1f0b444314a22b28c5242e58ed9 . "./code.cg2js/cg.fasl")
(#x731dfc4c135a6480cfc7a05ad6d1e193 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xe09e90e787a852024ad847e37ed7a113 . "./code.cg2js/cg-scaling-stream.fasl")
(#x3660ef0c597721eb17181f877d9ac6d9 . "./code.cg2js/cg-yes-no-string.fasl")
(#xbc5bcd6dcbe220e07deebead975436bd . "./code.cg2js/cg-icon-pixmap.fasl")
(#x198871528465970df2aba90267e32103 . "./code.cg2js/cg-web-browser.fasl")
(#xe8c5d1421e47841f53ed6d26466a8269 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x7969166c08bcdebaa48aa556f4420ac5 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x9da3f4c7283d455d678b97b334c9f6f4 . "./code.cg2js/cg-message-dialog.fasl")
(#x16700192dfa257a84f63f4b672dc673f . "./code.cg2js/cg-comtab.fasl")
(#x50eb561bd2352d9148d55cd68c3f8535 . "./code.cg2js/cg-base.fasl")
(#x9392dc37a05f36e2cccab5762c1cb0bd . "./code.cg2js/cg-pixmap-widget.fasl")
(#x3bd5b3d7fb162863dd2126d16aa2179f . "./code.cg2js/cg-text-edit-pane.fasl")
(#x1f0d527231fcdd507ebcafc114e579b4 . "./code.cg2js/cg-utility-dialog.fasl")
(#x4353aa874183350ccac6048213233fdf . "./code.cg2js/cg-object-editor.fasl")
(#x707fa2ce0f76b97297209a12f77930ea . "./code.cg2js/cg-edit-in-place.fasl")
(#x4c575c6fc3d4b12caca48fec1e7f0d3d . "./code.cg2js/cg-radio-button.fasl")
(#x694e0650f1e61ed22acb3fac50abb884 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xdc812aecd099c0be348c3d9fee887ae0 . "./code.cg2js/cg-nodes.fasl")
(#x05d016de040ce8ea4a926d4aa5f704c7 . "./code.cg2js/cg-get-position.fasl")
(#x92360544e3e75f52099e19caddd4f3f2 . "./code.cg2js/cg-directory-dialog.fasl")
(#x93cf116670d8ce2fab1824005a2361d6 . "./code.cg2js/cg-lisp-text.fasl")
(#xe3819dc60d1128e724209761e22cf2fa . "./code.cg2js/cg-combo-box.fasl")
(#xed5e499268a20cb11799e42986466a89 . "./code.cg2js/cg-text-widget.fasl")
(#x99a72034b8d102721c31ab89f1588747 . "./code.cg2js/cg-list-view.fasl")
(#x0340e159f1f6dbebbf89abe0732a5f7a . "./code.cg2js/cg-graphics-context.fasl")
(#x6bf532761949e3c4441c309db3646bc5 . "./code.cg2js/cg-get-pixmap.fasl")
(#x44ed920bdd2bb88e06230d7d18ae9c67 . "./code.cg2js/cg-menu.fasl")
(#x75385aa8efc721d222ade9e5ffd2b9a0 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xb92d4aa1b48a55617ea1b9fcd82c7c53 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x5ad512e27a51f43ff1e99d176ad23a5b . "./code.cg2js/cg-tooltip.fasl")
(#x705cfe3012890917d80565714ab6ee4f . "./code.cg2js/cg-html-widget.fasl")
(#xf80348e519bd3398855b833d8171c4c9 . "./code.cg2js/cg-choice-list.fasl")
(#x56b450e0a72afb31e3b1828a771e515a . "./code.cg2js/cg-icon.fasl")
(#x1cc7d32e259d0e593be3b4c2139845a8 . "./code.cg2js/cg-acache.fasl")
(#x24ba6cf0f1e24d313b0972db0200f2ed . "./code.cg2js/cg-template-string.fasl")
(#x5de7de337f8a5c577be49789a79832fb . "./code.cg2js/cg-bitmap-pane.fasl")
(#x26e54048d6d780b3b9004f6700be985b . "./code.cg2js/cg-toggling-widget.fasl")
(#x6f272896def34241da7bf50d85174989 . "./code.cg2js/cg-tab-control.fasl")
(#x4aa7746ba3430c1e07e7b395a2ef3c6f . "./code.cg2js/cg-split-bar.fasl")
(#x521feca9721ddc002978b2b36c2f08e5 . "./code.cg2js/cg-picture-widget.fasl")
(#xf1ee983e143bda60342c95afd2bdf0ec . "./code.cg2js/cg-player.fasl")
(#x782fb99f655818d74761f1f6212ef4d0 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xc13933bb0765a78aed8888018ca443b2 . "./code.cg2js/cg-choose-printer.fasl")
(#x0b87a310070cec3f3b18e34882b0d142 . "./code.cg2js/cg-os-window.fasl")
(#x6ea2c066c0b37d7eed15092a0505d4cc . "./code.cg2js/cg-scroll-bar.fasl")
(#x51fb7030e79c1ae1ee0fc6e544c02868 . "./code.cg2js/cg-clipboard.fasl")
(#x494a940985c89997014c193355af9b98 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe59217bd77dad71c882dffd14cabf840 . "./code.cg2js/cg-hotspot.fasl")
(#x293e7b3dcabdef80e82875790421b20d . "./code.cg2js/cg-check-box.fasl")
(#xb93a2ddc9bd925cbc93e0745468c9ce1 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x240f48075db5a9bbe21c5de99c5ba1e8 . "./code.cg2js/cg-file-dialog.fasl")
(#xd2f9595412a8318ce3a3842a98c46f85 . "./code.cg2js/cg-toolbar.fasl")
(#x735aaa77bad58efeaf2d4b095a39fc51 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x594251a17fcdfa0bf745a38550f04a7d . "./code.cg2js/cg-chart-or-plot.fasl")
(#xcd079d35465244466e11b4c025a313d3 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x20892b6aaa5462f0485b1212cc1e0022 . "./code.idejs/ide-project.fasl")
(#xb09574576b9fd5c5ce443959aa528d6a . "./code.idejs/ide-intro.fasl")
(#xb1b44f880e3bd51f65e625146e8d3ba2 . "./code.idejs/ide-find-in-files.fasl")
(#xab2b4791c0140faefb105c33aa64341a . "./code.idejs/ide-grapher.fasl")
(#xf47434136e71a9a06c82f664f92c12c0 . "./code.idejs/ide-inspector.fasl")
(#x6969f30e37535d2c9e64f5b41ef08676 . "./code.idejs/ide-editor.fasl")
(#xac5c413c835ea9c79a6159d5303eb516 . "./code.idejs/ide-window-list.fasl")
(#xf927c5563adc11c5bfaf0a3aea6b440b . "./code.idejs/ide-process-browser.fasl")
(#x119e53a798de63bb7e9614be2ad2dba9 . "./code.idejs/ide-patch-dialog.fasl")
(#xeca8219c6e727b57de693297ba684c4f . "./code.idejs/ide-class-grapher.fasl")
(#x134be91167e8c6b7d2c20284671f3978 . "./code.idejs/ide-base.fasl")
(#x0989eadb4cbddf69b819be10945a276b . "./code.idejs/ide.fasl")
(#x591985ad4d0ee93a1f451ef8d3950a6a . "./code.idejs/ide-trace.fasl")
(#x1e8304bbf00e08d7a4e6a4ffb841cba9 . "./code.idejs/ide-form.fasl")
(#xf91cd73426e5d2ef7e2746fa81fd3c84 . "./code.idejs/ide-find-definitions.fasl")
(#x3eda2dffe5fefb845b77c2d5a375c268 . "./code.idejs/ide-apropos.fasl")
(#x2d46f5cfef87f68ddc9053273b9b6e1e . "./code.idejs/ide-class-browser.fasl")
(#x35d3199ee9df16e91c22a019bda2d104 . "./code.idejs/ide-navigator.fasl")
(#xc5d6a6483eb5635d24af75c6f50c7796 . "./code.idejs/ide-clipboard.fasl")
(#xc25dd446537c7050925d081d4b01151c . "./code.idejs/ide-acache.fasl")
(#x29184267bdf8f3fd7b379a6e8e10d4db . "./code.idejs/ide-keyshort.fasl")
(#xfd7302065f0f5c369d771cc20f9e2f32 . "./code.idejs/ide-package-list.fasl")
(#x693d48469cae59ecc51d05595de4967f . "./code.idejs/ide-dev.fasl")
(#x4a9bad5a7becdca3247e09ac504560a1 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
